<?php
/** miniOrange enables user to log in using LDAP credentials.
    Copyright (C) 2015  miniOrange

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
* @package 		miniOrange OAuth
* @license		http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/
/**
This library is miniOrange Authentication Service. 
Contains Request Calls to LDAP Service.

**/
class Mo_Ldap_Config{
	
	function ldap_login($username, $password) {
		if(!Mo_Ldap_Util::is_curl_installed()) {
			return 'CURL_ERROR';
		}else if(!Mo_Ldap_Util::is_extension_installed('openssl')) {
			return 'OPENSSL_ERROR';
		}
		
		$url = get_option('mo_ldap_host_name')  . "/moas/api/ldap/authenticate";
		error_log('Sending request to ' . $url);
		global $post;
		//Send cURL request to gateway url and parse response
		
		//calls to encrypt username and password
		$encrypted_username = Mo_Ldap_Util::encrypt($username);
		$encrypted_password = Mo_Ldap_Util::encrypt($password);
		
		//Authorization Header
		/* The customer Key provided to you */
		$customerKey = get_option ( 'mo_ldap_admin_customer_key' );
		
		/* The customer API Key provided to you */
		$apiKey = get_option ( 'mo_ldap_admin_api_key' );
		
		/* Current time in milliseconds since midnight, January 1, 1970 UTC. */
		$currentTimeInMillis = round ( microtime ( true ) * 1000 );
		
		/* Creating the Hash using SHA-512 algorithm */
		$stringToHash = $customerKey . number_format ( $currentTimeInMillis, 0, '', '' ) . $apiKey;
		$hashValue = hash ( "sha512", $stringToHash );
		
		$customerKeyHeader = "Customer-Key: " . $customerKey;
		$timestampHeader = "Timestamp: " . number_format ( $currentTimeInMillis, 0, '', '' );
		$authorizationHeader = "Authorization: " . $hashValue;
		
		//send post request to gateway url
		$data = $this->get_login_config($encrypted_username, $username, $encrypted_password, 'User Login through LDAP', null);
		$data_string = json_encode($data);
		$curl = curl_init();
		
		curl_setopt_array($curl, array(
			CURLOPT_RETURNTRANSFER => 1,
			CURLOPT_URL => $url,
			CURLOPT_POST => 1,
			CURLOPT_POSTFIELDS => $data_string,
			CURLOPT_FOLLOWLOCATION => 1,
			CURLOPT_SSL_VERIFYHOST => false,
			CURLOPT_SSL_VERIFYPEER => true,
			CURLOPT_HTTPHEADER => array(                                                                          
				"Content-Type: application/json",
				$customerKeyHeader,
				$timestampHeader,
				$authorizationHeader,                                                                                
				'Content-Length: ' . strlen($data_string))
			));
		$response = curl_exec($curl);
		if (curl_errno($curl)) {
			   print curl_error($curl);
			   exit(0);
		} 
		$decoded_response = (array)json_decode($response);

		curl_close($curl);
		
		return $decoded_response;
	}
	
	function save_ldap_config(){
		if(!Mo_Ldap_Util::is_curl_installed()) {
			return json_encode(array("statusCode"=>'CURL_ERROR','statusMessage'=>'<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.'));
		}
		
		$url = get_option('mo_ldap_host_name') . '/moas/api/ldap/update-config';
		
		//Authorization Header
		/* The customer Key provided to you */
		$customerKey = get_option ( 'mo_ldap_admin_customer_key' );
		
		/* The customer API Key provided to you */
		$apiKey = get_option ( 'mo_ldap_admin_api_key' );
		
		/* Current time in milliseconds since midnight, January 1, 1970 UTC. */
		$currentTimeInMillis = round ( microtime ( true ) * 1000 );
		
		/* Creating the Hash using SHA-512 algorithm */
		$stringToHash = $customerKey . number_format ( $currentTimeInMillis, 0, '', '' ) . $apiKey;
		$hashValue = hash ( "sha512", $stringToHash );
		
		$customerKeyHeader = "Customer-Key: " . $customerKey;
		$timestampHeader = "Timestamp: " . number_format ( $currentTimeInMillis, 0, '', '' );
		$authorizationHeader = "Authorization: " . $hashValue;
		
		$ch = curl_init($url);
		
		$fields = $this->get_encrypted_config('Save LDAP Configuration', null);
		$field_string = json_encode($fields);
		
		curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
		curl_setopt( $ch, CURLOPT_ENCODING, "" );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_AUTOREFERER, true );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, true );    # required for https urls
		curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 5);
		curl_setopt( $ch, CURLOPT_TIMEOUT, 20);
		curl_setopt( $ch, CURLOPT_MAXREDIRS, 10 );
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
			'Content-Type: application/json',
			$customerKeyHeader,
			$timestampHeader,
			$authorizationHeader,
			'charset: UTF - 8',
		));
		curl_setopt( $ch, CURLOPT_POST, true);
		curl_setopt( $ch, CURLOPT_POSTFIELDS, $field_string);
		$content = curl_exec($ch);
		
		if(curl_errno($ch)){
			$error = curl_error($ch);
			if(curl_errno($ch) == 28) {
				$error = 'Connection request timed out';
			}
			curl_close($ch);
			return json_encode(array("statusCode"=>'ERROR','statusMessage'=>$error . '. Please check your configuration. Also check troubleshooting under LDAP configuration.'));
		} else {
			curl_close($ch);
			return $content;
		}
	}
	
	/*
	*	Test connection for default config or user config
	*/
	function test_connection($is_default) {
		if(!Mo_Ldap_Util::is_curl_installed()) {
			return json_encode(array("statusCode"=>'CURL_ERROR','statusMessage'=>'<a target="_blank" href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.'));
		}
		
		$url = '';
		$request_type = '';
		
		//Check if request is for default connection
		if(Mo_Ldap_Util::check_empty_or_null($is_default)) {
			$url = get_option('mo_ldap_host_name') . "/moas/api/ldap/test";
			$request_type = 'Test User Connection';
		} else {
			$url = get_option('mo_ldap_host_name') . "/moas/api/ldap/test-default";
			$request_type = 'Test Default Connection';
		}
		
		//Authorization Header
		/* The customer Key provided to you */
		$customerKey = get_option ( 'mo_ldap_admin_customer_key' );
		
		/* The customer API Key provided to you */
		$apiKey = get_option ( 'mo_ldap_admin_api_key' );
		
		/* Current time in milliseconds since midnight, January 1, 1970 UTC. */
		$currentTimeInMillis = round ( microtime ( true ) * 1000 );
		
		/* Creating the Hash using SHA-512 algorithm */
		$stringToHash = $customerKey . number_format ( $currentTimeInMillis, 0, '', '' ) . $apiKey;
		$hashValue = hash ( "sha512", $stringToHash );
		
		$customerKeyHeader = "Customer-Key: " . $customerKey;
		$timestampHeader = "Timestamp: " . number_format ( $currentTimeInMillis, 0, '', '' );
		$authorizationHeader = "Authorization: " . $hashValue;
		
		$ch = curl_init($url);
		
		$fields = $this->get_encrypted_config($request_type, $is_default);
		$field_string = json_encode($fields);
		
		curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
		curl_setopt( $ch, CURLOPT_ENCODING, "" );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_AUTOREFERER, true );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, true );    # required for https urls
		curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 5);
		curl_setopt( $ch, CURLOPT_TIMEOUT, 20);
		curl_setopt( $ch, CURLOPT_MAXREDIRS, 10 );
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
			'Content-Type: application/json',
			'charset: UTF - 8',
			$customerKeyHeader,
			$timestampHeader,
			$authorizationHeader,
			));
		curl_setopt( $ch, CURLOPT_POST, true);
		curl_setopt( $ch, CURLOPT_POSTFIELDS, $field_string);
		$content = curl_exec($ch);
		
		if(curl_errno($ch)){
			$error = curl_error($ch);
			if(curl_errno($ch) == 28) {
				$error = 'Connection request timed out';
			}
			curl_close($ch);
			return json_encode(array("statusCode"=>'ERROR','statusMessage'=>$error . '. Please check your configuration. Also check troubleshooting under LDAP configuration.'));
		} else {
			curl_close($ch);
			return $content;
		}
	}
	
	/*
	*	Test authentication for default config or user config
	*/
	function test_authentication($username, $password, $is_default) {
		if(!Mo_Ldap_Util::is_curl_installed()) {
			return json_encode(array("statusCode"=>'CURL_ERROR','statusMessage'=>'<a href="http://php.net/manual/en/curl.installation.php">PHP cURL extension</a> is not installed or disabled.'));
		}else if(!Mo_Ldap_Util::is_extension_installed('openssl')) {
			return json_encode(array("statusCode"=>'OPENSSL_ERROR','statusMessage'=>'<a href="http://php.net/manual/en/mcrypt.installation.php">PHP mcrypt extension</a> is not installed or disabled.'));
		}
		
		$url = '';
		$request_type = '';
		
		//Check if request is for default auth
		if(Mo_Ldap_Util::check_empty_or_null($is_default)) {
			$url = get_option('mo_ldap_host_name') . "/moas/api/ldap/authenticate";
			$request_type = 'Test User Login';
			$encrypted_username = Mo_Ldap_Util::encrypt($username);
			$encrypted_password = Mo_Ldap_Util::encrypt($password);
		} else {
			$url = get_option('mo_ldap_host_name') . "/moas/api/ldap/authenticate-default";
			$request_type = 'Test Default Login';
			$encrypted_username = $username;
			$encrypted_password = $password;
		}

		
		//Authorization Header
		/* The customer Key provided to you */
		$customerKey = get_option ( 'mo_ldap_admin_customer_key' );
		
		/* The customer API Key provided to you */
		$apiKey = get_option ( 'mo_ldap_admin_api_key' );
		
		/* Current time in milliseconds since midnight, January 1, 1970 UTC. */
		$currentTimeInMillis = round ( microtime ( true ) * 1000 );
		
		/* Creating the Hash using SHA-512 algorithm */
		$stringToHash = $customerKey . number_format ( $currentTimeInMillis, 0, '', '' ) . $apiKey;
		$hashValue = hash ( "sha512", $stringToHash );
		
		$customerKeyHeader = "Customer-Key: " . $customerKey;
		$timestampHeader = "Timestamp: " . number_format ( $currentTimeInMillis, 0, '', '' );
		$authorizationHeader = "Authorization: " . $hashValue;
		
		$ch = curl_init($url);
		
		$fields = $this->get_login_config($encrypted_username, $username, $encrypted_password, $request_type, $is_default);
		$field_string = json_encode($fields);
		
		curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, true );
		curl_setopt( $ch, CURLOPT_ENCODING, "" );
		curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $ch, CURLOPT_AUTOREFERER, true );
		curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, true );    # required for https urls
		curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 5);
		curl_setopt( $ch, CURLOPT_TIMEOUT, 20);
		curl_setopt( $ch, CURLOPT_MAXREDIRS, 10 );
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
			'Content-Type: application/json',
			'charset: UTF - 8',
			$customerKeyHeader,
			$timestampHeader,
			$authorizationHeader,
			));
		curl_setopt( $ch, CURLOPT_POST, true);
		curl_setopt( $ch, CURLOPT_POSTFIELDS, $field_string);
		$content = curl_exec($ch);
		
		if(curl_errno($ch)){
			$error = curl_error($ch);
			if(curl_errno($ch) == 28) {
				$error = 'Connection request timed out';
			}
			curl_close($ch);
			return json_encode(array("statusCode"=>'ERROR','statusMessage'=>$error . '. Please check your configuration. Also check troubleshooting under LDAP configuration.'));
		} else {
			curl_close($ch);
			return $content;
		}
	}
	
	function get_encrypted_config($request_type, $is_default) {
		global $current_user;
		$current_user = wp_get_current_user();
		
		$server_name = '';
		$dn = '';
		$admin_ldap_password = '';
		$dn_attribute = '';
		$search_base = '';
		$search_filter = '';
		$username = $current_user->user_email;
		
		if(Mo_Ldap_Util::check_empty_or_null($is_default)) {
			$server_name = get_option( 'mo_ldap_server_url');
			$dn = get_option( 'mo_ldap_server_dn');
			$admin_ldap_password = get_option( 'mo_ldap_server_password');
			//$dn_attribute = get_option( 'mo_ldap_dn_attribute');
			$search_base = get_option( 'mo_ldap_search_base');
			$search_filter = get_option( 'mo_ldap_search_filter');
			$username = get_option('mo_ldap_admin_email');
		}
		$customer_id = get_option('mo_ldap_admin_customer_key') ? get_option('mo_ldap_admin_customer_key') : null;
		
		$fields = array(
			'customerId' => $customer_id,
			'ldapAuditRequest' => array(
				'endUserEmail' => $username,
				'applicationName' => $_SERVER['SERVER_NAME'],
				'appType' => 'WP LDAP for Cloud',
				'requestType' => $request_type
			),
			'gatewayConfiguration' => array(
				'ldapServer' =>$server_name,
				'bindAccountDN'=>$dn,
				'bindAccountPassword'=>$admin_ldap_password,
				'searchBase'=>$search_base,
				'dnAttribute'=>$dn_attribute,
				'ldapSearchFilter'=>$search_filter
			)
		);
		
		return $fields;
	}
	
	function get_login_config($encrypted_username, $username, $encrypted_password, $request_type, $is_default) {
		global $current_user;
		$current_user = wp_get_current_user();
		
		$customer_id = get_option('mo_ldap_admin_customer_key') ? get_option('mo_ldap_admin_customer_key') : null;
		
		$profileAttributes = "";
		if(get_option('mo_ldap_email_attribute'))
			$profileAttributes = $profileAttributes.get_option('mo_ldap_email_attribute').",";
		if(get_option('mo_ldap_phone_attribute'))
			$profileAttributes = $profileAttributes.get_option('mo_ldap_phone_attribute').",";
		if(get_option('mo_ldap_fname_attribute'))
			$profileAttributes = $profileAttributes.get_option('mo_ldap_fname_attribute').",";
		if(get_option('mo_ldap_lname_attribute'))
			$profileAttributes = $profileAttributes.get_option('mo_ldap_lname_attribute').",";
		
		$wp_options = wp_load_alloptions();
		foreach($wp_options as $option=>$value){
			if(strpos($option, "mo_ldap_custom_attribute") === false){
			}else{
				$profileAttributes = $profileAttributes.$value.",";
			}
		}
					
		$fields = array(
			'customerId' => $customer_id,
			'userName' => $encrypted_username,
			'password' => $encrypted_password,
			'ldapAuditRequest' => array(
				'endUserEmail' => $username,
				'applicationName' => $_SERVER['SERVER_NAME'],
				'appType' => 'WP LDAP for Cloud',
				'requestType' => $request_type
			),
			'profileAttributes' => $profileAttributes
		);
		
		return $fields;
	}
	
	
	function test_attribute_configuration($username){
	if(!Mo_Ldap_Util::is_extension_installed('openssl')) {
			return null;
		}

	$server = get_option('mo_ldap_server_url') ? Mo_Ldap_Util::decrypt(get_option('mo_ldap_server_url')) : '';
	$ldap_bind_dn = get_option('mo_ldap_server_dn') ? Mo_Ldap_Util::decrypt(get_option('mo_ldap_server_dn')) : '';
	$ldap_bind_password = get_option('mo_ldap_server_password') ? Mo_Ldap_Util::decrypt(get_option('mo_ldap_server_password')) : '';
	$search_base_string = get_option('mo_ldap_search_base') ? Mo_Ldap_Util::decrypt(get_option('mo_ldap_search_base')) : '';
	$search_base = explode(";", $search_base_string);
	$search_filter = get_option('mo_ldap_search_filter') ? Mo_Ldap_Util::decrypt(get_option('mo_ldap_search_filter')) : '';
	$search_filter = str_replace('?', $username, $search_filter);
		
		$email_attribute = strtolower(get_option('mo_ldap_email_attribute'));
		$phone_attribute = strtolower(get_option('mo_ldap_phone_attribute'));
		$first_name_attribute = strtolower(get_option('mo_ldap_fname_attribute'));
		$last_name_attribute = strtolower(get_option('mo_ldap_lname_attribute'));
        $nickname_attribute = strtolower(get_option('mo_ldap_nickname_attribute'));

		$attr = array($email_attribute, $phone_attribute, $first_name_attribute, $last_name_attribute, $nickname_attribute);

		$wp_options = wp_load_alloptions();
		foreach($wp_options as $option=>$value){
			if(strpos($option, "mo_ldap_custom_attribute") === false){
				//Do nothing
			} else{
				array_push($attr, $value);
			}
		}
		if ( version_compare(PHP_VERSION, '5.3.0') >= 0 ) {
			ldap_set_option( null, LDAP_OPT_NETWORK_TIMEOUT, 5);
		}
		$ldapconn = ldap_connect($server);
		if ($ldapconn) { 
			ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3);
			ldap_set_option($ldapconn, LDAP_OPT_REFERRALS, 0);
			$bind = @ldap_bind($ldapconn, $ldap_bind_dn, $ldap_bind_password);
	          if($bind){
					for($i = 0 ; $i < sizeof($search_base) ; $i++){
						if(ldap_search($ldapconn, $search_base[$i], $search_filter, $attr))
							$user_search_result = ldap_search($ldapconn, $search_base[$i], $search_filter, $attr);
							$info = ldap_first_entry($ldapconn, $user_search_result);
							$entry = ldap_get_entries($ldapconn, $user_search_result);
							
							if($info){
								$dn = ldap_get_dn($ldapconn, $info);
								break;
							}
						else{
							//Do nothing
						}
					}
				}
	
		?>
		<style>
					table { border-collapse: collapse; width: 100%; }
					table, th, td { border: 1px solid black;}
					td{padding:5px;}
				</style>
				<h2>Attribute Mapping Test : </h2>
				<div style="font-family:Calibri;padding:0 3%;">
					<?php if(!empty($dn)){ ?>
						<div style="color: #3c763d;background-color: #dff0d8; padding:2%;margin-bottom:20px;text-align:center; border:1px solid #AEDB9A; font-size:18pt;">TEST SUCCESSFUL</div>
						<div style="display:block;text-align:center;margin-bottom:4%;"><img style="width:15%;"src="<?php echo plugin_dir_url(__FILE__) . 'includes/images/green_check.png'; ?>"/></div>
					<?php } else { ?>
						<div style="color: #a94442;background-color: #f2dede;padding: 15px;margin-bottom: 20px;text-align:center;border:1px solid #E6B3B2;font-size:18pt;">TEST FAILED</div>
						<div style="color: #a94442;font-size:14pt; margin-bottom:20px;">WARNING: Some Attributes Did Not Match.</div>
						<div style="display:block;text-align:center;margin-bottom:4%;"><img style="width:15%;"src="<?php echo plugin_dir_url(__FILE__) . 'includes/images/wrong.png'; ?>"/></div>
					<?php } ?>
				<table>
					<tr>
						<td style='font-weight:bold;border:2px solid #949090;padding:2%;'>User DN</td>
						<td style='padding:2%;border:2px solid #949090; word-wrap:break-word;'><?php echo isset($dn) ? $dn: '<b style="color:red;">No such attribute found.</b>'; ?></td>
					</tr>
					<?php
						foreach($attr as $attribute){
							?><tr>
								<td style='font-weight:bold;border:2px solid #949090;padding:2%;'><?php echo $attribute; ?></td>
								<td style='padding:2%;border:2px solid #949090; word-wrap:break-word;'>
								<?php 
									if(isset($entry[0][$attribute][0])){
										if(isset($entry[0][$attribute]['count'])){
											for($i=0;$i<$entry[0][$attribute]['count'];$i++)
												echo $entry[0][$attribute][$i]."<br>";
										}
										else
											echo $entry[0][$attribute][0];
									} else
										echo '<b style="color:red;">No such attribute found.</b>'; 
									?>
								</td>
							</tr>
						<?php
					}
					?>

				</table>
				<div style="margin:3%;display:block;text-align:center;"><input style="padding:1%;width:100px;background: #0091CD none repeat scroll 0% 0%;cursor: pointer;font-size:15px;border-width: 1px;border-style: solid;border-radius: 3px;white-space: nowrap;box-sizing: border-box;border-color: #0073AA;box-shadow: 0px 1px 0px rgba(120, 200, 230, 0.6) inset;color: #FFF;"type="button" value="Done" onClick="self.close();"/></div>

			<?php exit();
			//}
		
}
	}
	
}?>